@@ -1,4 +1,4 @@
-/* $OpenBSD: ecparam.c,v 1.8 2014/12/28 15:48:52 jsing Exp $ */
+/* $OpenBSD: ecparam.c,v 1.10 2015/07/20 03:02:46 doug Exp $ */
 /*
  * Written by Nils Larsch for the OpenSSL project.
  */
@@ -406,8 +406,6 @@ ecparam_main(int argc, char **argv)
 			goto end;
 	}
 	if (ecparam_config.check) {
-		if (group == NULL)
-			BIO_printf(bio_err, "no elliptic curve parameters\n");
 		BIO_printf(bio_err, "checking elliptic curve parameters: ");
 		if (!EC_GROUP_check(group, NULL)) {
 			BIO_printf(bio_err, "failed\n");
@@ -453,10 +451,6 @@ ecparam_main(int argc, char **argv)
 		if (!EC_GROUP_get_cofactor(group, ec_cofactor, NULL))
 			goto end;
 
-		if (!ec_p || !ec_a || !ec_b || !ec_gen ||
-		    !ec_order || !ec_cofactor)
-			goto end;
-
 		len = BN_num_bits(ec_order);
 
 		if ((tmp_len = (size_t) BN_num_bytes(ec_p)) > buf_len)
