@@ -1,4 +1,4 @@
-/* $OpenBSD: genpkey.c,v 1.12 2014/07/12 17:54:31 jsing Exp $ */
+/* $OpenBSD: genpkey.c,v 1.2 2014/08/28 14:23:52 jsing Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2006
  */
@@ -97,8 +97,6 @@ genpkey_main(int argc, char **argv)
 
 	outformat = FORMAT_PEM;
 
-	ERR_load_crypto_strings();
-	OpenSSL_add_all_algorithms();
 	args = argv + 1;
 	while (!badarg && *args && *args[0] == '-') {
 		if (!strcmp(*args, "-outform")) {
